@@ -15,6 +15,7 @@
 #include "soup.h"
 #include "soup-connection.h"
 #include "soup-message-private.h"
+#include "soup-message-headers-private.h"
 #include "soup-message-metrics-private.h"
 #include "soup-uri-utils-private.h"
 #include "content-sniffer/soup-content-sniffer-stream.h"
@@ -89,11 +90,16 @@ typedef struct {
 	GTlsCertificate      *tls_peer_certificate;
 	GTlsCertificateFlags  tls_peer_certificate_errors;
 
+        GTlsCertificate *tls_client_certificate;
+        GTask *pending_tls_cert_request;
+        GTask *pending_tls_cert_pass_request;
+
 	SoupMessagePriority priority;
 
 	gboolean is_top_level_navigation;
         gboolean is_options_ping;
         gboolean force_http1;
+        gboolean is_misdirected_retry;
         guint    last_connection_id;
         GSocketAddress *remote_address;
 
@@ -119,6 +125,8 @@ enum {
 	AUTHENTICATE,
 	NETWORK_EVENT,
 	ACCEPT_CERTIFICATE,
+        REQUEST_CERTIFICATE,
+        REQUEST_CERTIFICATE_PASSWORD,
 	HSTS_ENFORCED,
 
 	LAST_SIGNAL
@@ -169,6 +177,16 @@ soup_message_finalize (GObject *object)
 	SoupMessage *msg = SOUP_MESSAGE (object);
 	SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
 
+        if (priv->pending_tls_cert_request) {
+                g_task_return_int (priv->pending_tls_cert_request, G_TLS_INTERACTION_FAILED);
+                g_object_unref (priv->pending_tls_cert_request);
+        }
+
+        if (priv->pending_tls_cert_pass_request) {
+                g_task_return_int (priv->pending_tls_cert_pass_request, G_TLS_INTERACTION_FAILED);
+                g_object_unref (priv->pending_tls_cert_pass_request);
+        }
+
 	soup_message_set_connection (msg, NULL);
 
 	g_clear_pointer (&priv->uri, g_uri_unref);
@@ -183,6 +201,7 @@ soup_message_finalize (GObject *object)
 
 	g_clear_object (&priv->tls_peer_certificate);
         g_clear_object (&priv->remote_address);
+        g_clear_object (&priv->tls_client_certificate);
 
 	soup_message_headers_unref (priv->request_headers);
 	soup_message_headers_unref (priv->response_headers);
@@ -594,6 +613,65 @@ soup_message_class_init (SoupMessageClass *message_class)
 			      G_TYPE_TLS_CERTIFICATE,
 			      G_TYPE_TLS_CERTIFICATE_FLAGS);
 
+        /**
+         * SoupMessage::request-certificate:
+         * @msg: the message
+         * @tls_connection: the #GTlsClientConnection
+         *
+         * Emitted during the @msg's connection TLS handshake when
+         * @tls_connection requests a certificate from the client.
+         * You can set the client certificate by calling
+         * soup_message_set_tls_client_certificate() and returning %TRUE.
+         * It's possible to handle the request asynchornously by returning
+         * %TRUE and call soup_message_set_tls_client_certificate() later
+         * once the certificate is available.
+         * Note that this signal is not emitted if #SoupSession::tls-interaction
+         * was set, or if soup_message_set_tls_client_certificate() was called
+         * before the connection TLS handshake started.
+         *
+         * Returns: %TRUE to handle the request, or %FALSE to make the connection
+         *     fail with %G_TLS_ERROR_CERTIFICATE_REQUIRED.
+         */
+        signals[REQUEST_CERTIFICATE] =
+                g_signal_new ("request-certificate",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              0,
+                              g_signal_accumulator_true_handled, NULL,
+                              NULL,
+                              G_TYPE_BOOLEAN, 1,
+                              G_TYPE_TLS_CLIENT_CONNECTION);
+
+        /**
+         * SoupMessage::request-certificate-password:
+         * @msg: the message
+         * @tls_password: the #GTlsPassword
+         *
+         * Emitted during the @msg's connection TLS handshake when
+         * @tls_connection requests a certificate password from the client.
+         * You can set the certificate password on @password, then call
+         * soup_message_tls_client_certificate_password_request_complete() and return %TRUE
+         * to handle the signal synchronously.
+         * It's possible to handle the request asynchornously by calling g_object_ref()
+         * on @password, then returning %TRUE and call
+         * soup_message_tls_client_certificate_password_request_complete() later after
+         * setting the password on @password.
+         * Note that this signal is not emitted if #SoupSession::tls-interaction
+         * was set.
+         *
+         * Returns: %TRUE to handle the request, or %FALSE to make the connection
+         *     fail with %G_TLS_ERROR_CERTIFICATE_REQUIRED.
+         */
+        signals[REQUEST_CERTIFICATE_PASSWORD] =
+                g_signal_new ("request-certificate-password",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_LAST,
+                              0,
+                              g_signal_accumulator_true_handled, NULL,
+                              NULL,
+                              G_TYPE_BOOLEAN, 1,
+                              G_TYPE_TLS_PASSWORD);
+
 	/**
 	 * SoupMessage::hsts-enforced:
 	 * @msg: the message
@@ -984,7 +1062,7 @@ soup_message_set_request_body (SoupMessage  *msg,
                         g_warn_if_fail (strchr (content_type, '/') != NULL);
 
                         if (soup_message_headers_get_content_type (priv->request_headers, NULL) != content_type)
-                                soup_message_headers_replace (priv->request_headers, "Content-Type", content_type);
+                                soup_message_headers_replace_common (priv->request_headers, SOUP_HEADER_CONTENT_TYPE, content_type);
                 }
 
                 if (content_length == -1)
@@ -994,8 +1072,8 @@ soup_message_set_request_body (SoupMessage  *msg,
 
                 priv->request_body_stream = g_object_ref (stream);
         } else {
-                soup_message_headers_remove (priv->request_headers, "Content-Type");
-                soup_message_headers_remove (priv->request_headers, "Content-Length");
+                soup_message_headers_remove_common (priv->request_headers, SOUP_HEADER_CONTENT_TYPE);
+                soup_message_headers_remove_common (priv->request_headers, SOUP_HEADER_CONTENT_LENGTH);
         }
 }
 
@@ -1391,6 +1469,40 @@ re_emit_accept_certificate (SoupMessage          *msg,
 	return accept;
 }
 
+static gboolean
+re_emit_request_certificate (SoupMessage          *msg,
+                             GTlsClientConnection *tls_conn,
+                             GTask                *task)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+        gboolean handled = FALSE;
+
+        priv->pending_tls_cert_request = g_object_ref (task);
+
+        g_signal_emit (msg, signals[REQUEST_CERTIFICATE], 0, tls_conn, &handled);
+        if (!handled)
+                g_clear_object (&priv->pending_tls_cert_request);
+
+        return handled;
+}
+
+static gboolean
+re_emit_request_certificate_password (SoupMessage  *msg,
+                                      GTlsPassword *password,
+                                      GTask        *task)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+        gboolean handled = FALSE;
+
+        priv->pending_tls_cert_pass_request = g_object_ref (task);
+
+        g_signal_emit (msg, signals[REQUEST_CERTIFICATE_PASSWORD], 0, password, &handled);
+        if (!handled)
+                g_clear_object (&priv->pending_tls_cert_pass_request);
+
+        return handled;
+}
+
 static void
 re_emit_tls_certificate_changed (SoupMessage    *msg,
 				 GParamSpec     *pspec,
@@ -1421,6 +1533,13 @@ soup_message_set_connection (SoupMessage    *msg,
 	if (priv->connection) {
 		g_signal_handlers_disconnect_by_data (priv->connection, msg);
                 priv->io_data = NULL;
+
+                if (priv->pending_tls_cert_request) {
+                        soup_connection_complete_tls_certificate_request (priv->connection,
+                                                                          priv->tls_client_certificate,
+                                                                          g_steal_pointer (&priv->pending_tls_cert_request));
+                        g_clear_object (&priv->tls_client_certificate);
+                }
 		g_object_remove_weak_pointer (G_OBJECT (priv->connection), (gpointer*)&priv->connection);
                 soup_connection_set_in_use (priv->connection, FALSE);
 	}
@@ -1438,12 +1557,24 @@ soup_message_set_connection (SoupMessage    *msg,
                                                soup_connection_get_tls_certificate_errors (priv->connection));
         soup_message_set_remote_address (msg, soup_connection_get_remote_address (priv->connection));
 
+        if (priv->tls_client_certificate) {
+                soup_connection_set_tls_client_certificate (priv->connection,
+                                                            priv->tls_client_certificate);
+                g_clear_object (&priv->tls_client_certificate);
+        }
+
 	g_signal_connect_object (priv->connection, "event",
 				 G_CALLBACK (re_emit_connection_event),
 				 msg, G_CONNECT_SWAPPED);
 	g_signal_connect_object (priv->connection, "accept-certificate",
 				 G_CALLBACK (re_emit_accept_certificate),
 				 msg, G_CONNECT_SWAPPED);
+        g_signal_connect_object (priv->connection, "request-certificate",
+                                 G_CALLBACK (re_emit_request_certificate),
+                                 msg, G_CONNECT_SWAPPED);
+        g_signal_connect_object (priv->connection, "request-certificate-password",
+                                 G_CALLBACK (re_emit_request_certificate_password),
+                                 msg, G_CONNECT_SWAPPED);
 	g_signal_connect_object (priv->connection, "notify::tls-certificate",
 				 G_CALLBACK (re_emit_tls_certificate_changed),
 				 msg, G_CONNECT_SWAPPED);
@@ -1686,15 +1817,18 @@ soup_message_is_keepalive (SoupMessage *msg)
 		 * doesn't request it. So ignore c_conn.
 		 */
 
-		if (!soup_message_headers_header_contains (priv->response_headers,
-							   "Connection", "Keep-Alive"))
+		if (!soup_message_headers_header_contains_common (priv->response_headers,
+                                                                  SOUP_HEADER_CONNECTION,
+                                                                  "Keep-Alive"))
 			return FALSE;
 	} else {
 		/* Normally persistent unless either side requested otherwise */
-		if (soup_message_headers_header_contains (priv->request_headers,
-							  "Connection", "close") ||
-		    soup_message_headers_header_contains (priv->response_headers,
-							  "Connection", "close"))
+		if (soup_message_headers_header_contains_common (priv->request_headers,
+                                                                 SOUP_HEADER_CONNECTION,
+                                                                 "close") ||
+		    soup_message_headers_header_contains_common (priv->response_headers,
+                                                                 SOUP_HEADER_CONNECTION,
+                                                                 "close"))
 			return FALSE;
 
 		return TRUE;
@@ -2100,6 +2234,76 @@ soup_message_get_tls_peer_certificate_errors (SoupMessage *msg)
 	return priv->tls_peer_certificate_errors;
 }
 
+/**
+ * soup_message_set_tls_client_certificate:
+ * @msg: a #SoupMessage
+ * @certificate: the #GTlsCertificate to set
+ *
+ * Sets the @certificate to be used by @msg's connection when a
+ * client certificate is requested during the TLS handshake.
+ * You can call this as a response to #SoupMessage::request-certificate
+ * signal, or before the connection is started.
+ * Note that the #GTlsCertificate set by this function will be ignored if
+ * #SoupSession::tls-interaction is not %NULL.
+ */
+void
+soup_message_set_tls_client_certificate (SoupMessage     *msg,
+                                         GTlsCertificate *certificate)
+{
+        SoupMessagePrivate *priv;
+
+        g_return_if_fail (SOUP_IS_MESSAGE (msg));
+        g_return_if_fail (G_IS_TLS_CERTIFICATE (certificate));
+
+        priv = soup_message_get_instance_private (msg);
+        if (priv->pending_tls_cert_request) {
+                g_assert (SOUP_IS_CONNECTION (priv->connection));
+                soup_connection_complete_tls_certificate_request (priv->connection,
+                                                                  certificate,
+                                                                  g_steal_pointer (&priv->pending_tls_cert_request));
+                return;
+        }
+
+        if (priv->connection) {
+                soup_connection_set_tls_client_certificate (priv->connection,
+                                                            certificate);
+                return;
+        }
+
+        if (priv->tls_client_certificate == certificate)
+                return;
+
+        g_clear_object (&priv->tls_client_certificate);
+        priv->tls_client_certificate = g_object_ref (certificate);
+}
+
+/**
+ * soup_message_tls_client_certificate_password_request_complete:
+ * @msg: a #SoupMessage
+ *
+ * Completes a certificate password request.
+ *
+ * You must call this as a response to #SoupMessage::request-certificate-password
+ * signal, to notify @msg that the #GTlsPassword has already been updated.
+ */
+void
+soup_message_tls_client_certificate_password_request_complete (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv;
+
+        g_return_if_fail (SOUP_IS_MESSAGE (msg));
+
+        priv = soup_message_get_instance_private (msg);
+        if (!priv->pending_tls_cert_pass_request) {
+                g_warning ("soup_message_tls_client_certificate_password_request_complete should only be called as a response to SoupMessage::request-certificate-password signal");
+                return;
+        }
+
+        g_assert (SOUP_IS_CONNECTION (priv->connection));
+        soup_connection_complete_tls_certificate_password_request (priv->connection,
+                                                                   g_steal_pointer (&priv->pending_tls_cert_pass_request));
+}
+
 /**
  * SoupMessagePriority:
  * @SOUP_MESSAGE_PRIORITY_VERY_LOW: The lowest priority, the messages
@@ -2290,6 +2494,17 @@ soup_message_io_skip (SoupMessage  *msg,
         return soup_client_message_io_skip (priv->io_data, msg, blocking, cancellable, error);
 }
 
+GCancellable *
+soup_message_io_get_cancellable (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        if (!priv->io_data)
+                return NULL;
+
+        return soup_client_message_io_get_cancellable (priv->io_data, msg);
+}
+
 void
 soup_message_send_item (SoupMessage              *msg,
                         SoupMessageQueueItem     *item,
@@ -2323,6 +2538,14 @@ soup_message_set_content_sniffer (SoupMessage *msg, SoupContentSniffer *sniffer)
 	priv->sniffer = sniffer ? g_object_ref (sniffer) : NULL;
 }
 
+gboolean
+soup_message_has_content_sniffer (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        return priv->sniffer != NULL;
+}
+
 gboolean
 soup_message_try_sniff_content (SoupMessage  *msg,
                                 GInputStream *stream,
@@ -2650,12 +2873,12 @@ soup_message_set_request_host_from_uri (SoupMessage *msg,
 
         host = soup_uri_get_host_for_headers (uri);
         if (soup_uri_uses_default_port (uri))
-                soup_message_headers_replace (priv->request_headers, "Host", host);
+                soup_message_headers_replace_common (priv->request_headers, SOUP_HEADER_HOST, host);
         else {
                 char *value;
 
                 value = g_strdup_printf ("%s:%d", host, g_uri_get_port (uri));
-                soup_message_headers_replace (priv->request_headers, "Host", value);
+                soup_message_headers_replace_common (priv->request_headers, SOUP_HEADER_HOST, value);
                 g_free (value);
         }
         g_free (host);
@@ -2669,7 +2892,7 @@ soup_message_update_request_host_if_needed (SoupMessage *msg)
         if (priv->http_version == SOUP_HTTP_2_0)
                 return;
 
-        if (soup_message_headers_get_one (priv->request_headers, "Host"))
+        if (soup_message_headers_get_one_common (priv->request_headers, SOUP_HEADER_HOST))
                 return;
 
         soup_message_set_request_host_from_uri (msg, priv->uri);
@@ -2688,10 +2911,10 @@ soup_message_force_keep_alive_if_needed (SoupMessage *msg)
          * a short period of time, as we wouldn't need to establish
          * new connections. Keep alive is implicit for HTTP 1.1.
          */
-        if (!soup_message_headers_header_contains (priv->request_headers, "Connection", "Keep-Alive") &&
-            !soup_message_headers_header_contains (priv->request_headers, "Connection", "close") &&
-            !soup_message_headers_header_contains (priv->request_headers, "Connection", "Upgrade")) {
-                soup_message_headers_append (priv->request_headers, "Connection", "Keep-Alive");
+        if (!soup_message_headers_header_contains_common (priv->request_headers, SOUP_HEADER_CONNECTION, "Keep-Alive") &&
+            !soup_message_headers_header_contains_common (priv->request_headers, SOUP_HEADER_CONNECTION, "close") &&
+            !soup_message_headers_header_contains_common (priv->request_headers, SOUP_HEADER_CONNECTION, "Upgrade")) {
+                soup_message_headers_append_common (priv->request_headers, SOUP_HEADER_CONNECTION, "Keep-Alive");
         }
 }
 
@@ -2711,3 +2934,20 @@ soup_message_get_force_http1 (SoupMessage *msg)
 
         return priv->force_http1;
 }
+
+void
+soup_message_set_is_misdirected_retry (SoupMessage *msg,
+                                       gboolean     is_misdirected_retry)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        priv->is_misdirected_retry = is_misdirected_retry;
+}
+
+gboolean
+soup_message_is_misdirected_retry (SoupMessage *msg)
+{
+        SoupMessagePrivate *priv = soup_message_get_instance_private (msg);
+
+        return priv->is_misdirected_retry;
+}
