@@ -242,6 +242,68 @@ do_cancellation_test (Test *test, gconstpointer data)
         g_main_context_unref (async_context);
 }
 
+static void
+do_one_cancel_after_send_request_test (SoupSession *session,
+                                       gboolean     reuse_cancellable,
+                                       gboolean     cancelled_by_session)
+{
+        SoupMessage *msg;
+        GCancellable *cancellable;
+        GInputStream *istream;
+        GOutputStream *ostream;
+        guint flags = SOUP_TEST_REQUEST_CANCEL_AFTER_SEND_FINISH;
+        GBytes *body;
+        GError *error = NULL;
+
+        if (cancelled_by_session)
+                flags |= SOUP_TEST_REQUEST_CANCEL_BY_SESSION;
+
+        msg = soup_message_new (SOUP_METHOD_GET, "https://127.0.0.1:5000/");
+        cancellable = g_cancellable_new ();
+        istream = soup_test_request_send (session, msg, cancellable, flags, &error);
+        g_assert_no_error (error);
+        g_assert_nonnull (istream);
+
+        /* If we use a new cancellable to read the stream
+         * it shouldn't fail with cancelled error.
+         */
+        if (!reuse_cancellable) {
+                g_object_unref (cancellable);
+                cancellable = g_cancellable_new ();
+        }
+        ostream = g_memory_output_stream_new_resizable ();
+        g_output_stream_splice (ostream, istream,
+                                G_OUTPUT_STREAM_SPLICE_CLOSE_SOURCE |
+                                G_OUTPUT_STREAM_SPLICE_CLOSE_TARGET,
+                                cancellable, &error);
+
+        if (reuse_cancellable || cancelled_by_session) {
+                g_assert_error (error, G_IO_ERROR, G_IO_ERROR_CANCELLED);
+                g_clear_error (&error);
+        } else {
+                g_assert_no_error (error);
+                body = g_memory_output_stream_steal_as_bytes (G_MEMORY_OUTPUT_STREAM (ostream));
+                g_assert_cmpstr (g_bytes_get_data (body, NULL), ==, "Hello world");
+                g_bytes_unref (body);
+        }
+
+        while (g_main_context_pending (NULL))
+		g_main_context_iteration (NULL, FALSE);
+
+        g_object_unref (cancellable);
+        g_object_unref (ostream);
+        g_object_unref (istream);
+        g_object_unref (msg);
+}
+
+static void
+do_cancellation_after_send_test (Test *test, gconstpointer data)
+{
+        do_one_cancel_after_send_request_test (test->session, TRUE, FALSE);
+        do_one_cancel_after_send_request_test (test->session, FALSE, FALSE);
+        do_one_cancel_after_send_request_test (test->session, FALSE, TRUE);
+}
+
 static void
 do_post_sync_test (Test *test, gconstpointer data)
 {
@@ -297,8 +359,7 @@ do_post_blocked_async_test (Test *test, gconstpointer data)
 {
         GMainContext *async_context = g_main_context_ref_thread_default ();
 
-        GInputStream *parent_stream = g_memory_input_stream_new ();
-        GInputStream *in_stream = soup_body_input_stream_http2_new (G_POLLABLE_INPUT_STREAM (parent_stream));
+        GInputStream *in_stream = soup_body_input_stream_http2_new ();
         soup_body_input_stream_http2_add_data (SOUP_BODY_INPUT_STREAM_HTTP2 (in_stream), (guint8*)"Part 1 -", 8);
 
         test->msg = soup_message_new (SOUP_METHOD_POST, "https://127.0.0.1:5000/echo_post");
@@ -307,10 +368,9 @@ do_post_blocked_async_test (Test *test, gconstpointer data)
         GBytes *response = NULL;
         soup_session_send_async (test->session, test->msg, G_PRIORITY_DEFAULT, NULL, on_send_complete, &response);
 
-        int iteration_count = 20;
         while (!response) {
                 // Let it iterate for a bit waiting on blocked data
-                if (iteration_count-- == 0) {
+                if (soup_body_input_stream_http2_is_blocked (SOUP_BODY_INPUT_STREAM_HTTP2 (in_stream))) {
                         soup_body_input_stream_http2_add_data (SOUP_BODY_INPUT_STREAM_HTTP2 (in_stream), (guint8*)" Part 2", 8);
                         soup_body_input_stream_http2_complete (SOUP_BODY_INPUT_STREAM_HTTP2 (in_stream));
                 }
@@ -323,7 +383,6 @@ do_post_blocked_async_test (Test *test, gconstpointer data)
                 g_main_context_iteration (async_context, FALSE);
 
         g_bytes_unref (response);
-        g_object_unref (parent_stream);
         g_object_unref (in_stream);
         g_main_context_unref (async_context);
         g_object_unref (test->msg);
@@ -869,8 +928,7 @@ do_sniffer_sync_test (Test *test, gconstpointer data)
         soup_session_add_feature_by_type (test->session, SOUP_TYPE_CONTENT_SNIFFER);
 
         do_one_sniffer_test (test->session, "https://127.0.0.1:5000/", 11, TRUE, NULL);
-        /* FIXME: large seems to be broken in sync mode */
-        /* do_one_sniffer_test (test->session, "https://127.0.0.1:5000/large", (1024 * 24) + 1, TRUE, NULL); */
+        do_one_sniffer_test (test->session, "https://127.0.0.1:5000/large", (1024 * 24) + 1, TRUE, NULL);
         do_one_sniffer_test (test->session, "https://127.0.0.1:5000/no-content", 0, FALSE, NULL);
 }
 
@@ -886,8 +944,6 @@ main (int argc, char **argv)
                 return 1;
         }
 
-        g_setenv ("SOUP_ENABLE_HTTP2", "1", TRUE);
-
         g_test_add ("/http2/basic/async", Test, NULL,
                     setup_session,
                     do_basic_async_test,
@@ -900,11 +956,11 @@ main (int argc, char **argv)
                     setup_session,
                     do_no_content_async_test,
                     teardown_session);
-        g_test_add ("/http2/large/async", Test, GINT_TO_POINTER (FALSE),
+        g_test_add ("/http2/large/async", Test, GINT_TO_POINTER (TRUE),
                     setup_session,
                     do_large_test,
                     teardown_session);
-        g_test_add ("/http2/large/sync", Test, GINT_TO_POINTER (TRUE),
+        g_test_add ("/http2/large/sync", Test, GINT_TO_POINTER (FALSE),
                     setup_session,
                     do_large_test,
                     teardown_session);
@@ -960,6 +1016,10 @@ main (int argc, char **argv)
                     setup_session,
                     do_cancellation_test,
                     teardown_session);
+        g_test_add ("/http2/cancellation-after-send", Test, NULL,
+                    setup_session,
+                    do_cancellation_after_send_test,
+                    teardown_session);
         g_test_add ("/http2/invalid-header", Test, NULL,
                     setup_session,
                     do_invalid_header_test,
